//+------------------------------------------------------------------+
//|   Steve Hopwood's London session breakout auto trading robot.mq4 |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+

/*+------------------------------------------------------------------+
 *
 * Version history:
 *
 * V7.2: [Squalou]: (2011.03.26)
 *     - the EA will open MARKET ORDER when price is too close to the pending entry price;
 *       in that condition, no pending entry was opened, and therefore no trade was triggered;
 *     - Added "Slippage" input in pips (it was 0);
 *       Slippage is also used to check if we can open a market order when price is too close for opening pending orders;
 *     - FIXED SL_pips bug in V7.0: SL_pips input wasn't used properly;
 *
 * V7.0: [Squalou]: (2011.03.25)
 *     - Added "ImportFromIndicator": if true, then session parameters are imported from the Indicator (as before, requires the Indicator to be running),
 *       if false, then you can set all the session parameters manually directly in the EA, rather than importing them from Global Variables set by the Indicator:
 *       StartTime,EndTime,SessionEndTime,
 *       MaxBoxSizeInPips,EntryFactor,TPFactor,SLpips,SLFactor,LevelsResizeFactor,
 *       MinExtentInPips,MaxExtentInPips
 *
 *     - Added "SLpips" and "SLFactor" inputs: (request from BERT505)
 *       when SLpips is >0, then the SL is set to this fix amount;
 *       otherwise, SL is a factor of the box size indicated by SLFactor;
 *
 * V6.0: [Squalou]: (2010.05.28)
 *     - Added "AutoMartingale" mode:
 *       the Martingale factor will be computed dynamically so that the next winning trade
 *       will recoup all past consecutive losses, plus the current TP.
 *
 * V5.2: [Squalou]: (2010.05.27)
 *     - fixed Martingale code: it was moving to the next mart factor at session opening 
 *       if last yesterday's trade was a loss (instead of keeping the same factor)...;
 *
 * V5.1: [Squalou]: (2010.05.27)
 *     - debugged and reworked completely the Martingale code;
 *     - optimized EA -- especially for backtesting (removed useless loops on history; don't print status when backtesting)
 *       (it's probably at least 10 times faster now...)
 *     - fixed count of lost trades: it was counting trades older than session start, and even counting-in trades of previous weeks(!)
 *
 * V5.0: [Squalou]: (2010.05.25)
 *     - Added Martingale code from Pat1; !!! STILL EXPERIMENTAL CODE !!!
 *     - FIX: EA should not trade outside of trading hours;
 *
 *     You can setup the Martingale multipliers sequence in the "martingale_sequence" input.
 *     The default sequence is set to "0,1,2,5,13,34,89,233,610,1597,4181".
 *     When the first element in the list is 0, the martingale is DISABLED.
 *     This is the default.
 *     When a trade is a loss, the next multiplier is used for the next trade.
 *     When it is a win, we come back to the first multiplier for the next trade.
 *     When the last multiplier is reached (9 consecutive losses with the default sequence), the multiplier will either remain to the last value (4181 in the default sequence), or come back to the first, depending on "RestartMartSeq" input:
 *     "RestartMartSeq" set to false (default) will keep last multiplier until a win.
 *     Set to true will come back to first multiplier.
 *
 *     - REMOVED change done in v4.0 that would close all open/pending trades when the EA is unloaded:
 *       MT4 "silently" unloads the EAs when you close the MT4 platform, which closes all your open trades as well...
 *       OPEN trades are now left untouched,
 *       but pending orders are now controlled by the "ClosePendingWhenUnload" input.
 *
 * V4.0: [Squalou]: (2010.05.20)
 *     - Added TRAILING STOP capability: 
 *       new inputs: 
 *          TrailingStopPips(0): pips to trail the StopLoss; when 0 no trailing is applied (fixed SL as before).
 *          TrailingStopStep(1): trailing SL will jump by this amount instead of trailing at every tick;
 *          (helps limit the number of sent orders, and therefore order rejections).
 *
 *     - Added profit lock-in at fixed TP:
 *       when "BreakEvenPips" is >0, SL is moved to BE+"BreakEvenProfitInPips" when price has reached BE+"BreakEvenPips";
 *       Works independently from "AllowHalfClose" option (but uses the same BreakEvenProfitInPips value);
 *       Will also be trailed if TrailingStop is selected;
 *       (inspired from Steve's MPTM EA...)
 *
 *     - Added MaxRisk input: if >0, then order size is the min of Lot input and calculated max Lot based on MaxRisk and Stop-Loss;
 *
 *     - All open/pending trades will be closed when the EA is unloaded;
 *
 *     - when MagicNumber is 0, a unique MagicNumber is created by the EA, MagicNumbers will be different for each pair/TF;
 *       This helps running the EA on different pairs/TF without having to 
 *
 *     - fix: in V3.0, objPrefix default value was not set to "LB-" as it should have been, leading to error 130 "invalid stops";
 *     - added Alerts when Indicator SETTINGS GlobalVariables cannot be found; EA will stop trading in this case.
 *
 * V3.0: [Squalou]: (2010.05.11)
 *       ========== ALLOWS BACKTESTING THE EA ==========
 *     - the EA now computes the Entry/TP/SL LEVELS by itself, and only imports Indicator SETTINGS
 *       from the GlobalVariables.
 *     - EA now starts trading at the END of the bar forming at the Box EndTime, as expected by the Strategy:
 *       on M15, with EndTime=07:00, EA will start trading at 07:15, not 07:01.
 *     - fixed recursive alarms playing when price goes out of the "green fields";
 *       Only valid pending STOP orders are now placed based on market price;
 *     - Fixed OCO function that was repeatedly trying to delete wrong orders;
 *     - fixed count of losses for the day: when EA was used on multiple pairs/charts,
 *       ALL running EAs would "stop trading for the day" when the FIRST one reached MaxLossesAllowedPerPair...
 *     - fixed Half-close not functionning on SELL side;
 *
 * V2.0: 
 *     - designed to work with "London BreakoutV8.1.mq4" indicator "exported" GobalVariables:
 *       grabs "MaxBoxSizeInPips", "StartTime", "EndTime", and "SessionEndTime"
 *
 * V1: original version from SteveHopwood 
 *     - designed to work with "London BreakoutV8.mq4" indicator "exported" GobalVariables:
 *
 *+------------------------------------------------------------------+
 */

#property copyright "Open source -- by SteveHopwood & Squalou"

#property show_inputs
#include <WinUser32.mqh>
#include <stdlib.mqh>


#define  EA_VERSION    "London Breakout_v1.02"

//+------------------------------------------------------------------+

extern double     Lot                            = 0.02;
extern double     MaxRisk                        = 0;      // if 0 then use fixed Lots above
extern int        MagicNumber                    = 0;      // if 0 then will autocreate a unique MagicNumber
extern string     TradeComment                   = "----London breakout";
extern string     _mta1                          = "----MaxTradesAllowedForTheDay = 0";
extern string     _mta2                          = "----means no limit on trades";
extern int        MaxTradesAllowedForTheDay      = 0;
extern string     _mla1                          = "----MaxLossesAllowedPerPair = 0";
extern string     _mla2                          = "----means no limit on losses";
extern int        MaxLossesAllowedPerPair        = 2;
extern string     objPrefix                      = "LB-";  // Prefixes of London Breakout Indicator objects;
extern string     _tm                            = "----Trade management----";
extern bool       UseSelectedManagementFunctions = true;
extern string     _hcf                           = "----Half-close function";
extern bool       AllowHalfClose                 = true;
extern int        BreakEvenPips                  = 0;      // when >0, will move SL to BE+BreakEvenProfitInPips when price has reached BE+BreakEvenPips
extern int        BreakEvenProfitInPips          = 5;      // lock-in this amount of pips when HalfClose is hit;
extern int        TrailingStopPips               = 0;      // Pips to trail the StopLoss; if 0 then no trailing stop
extern int        TrailingStopStep               = 1;      // trailing SL jumping step in pips
extern string     _oco1                          = "----OCO will delete remaining pending";
extern string     _oco2                          = "----when its partner fills";
extern bool       OneCancelsTheOther             = false;
extern bool       ClosePendingWhenUnload         = false;

extern string     _mart1                         = "----MARTINGALE multipliers sequence:";
extern string     _mart2                         = "----not used when first multiplier is 0:";
extern string     martingale_sequence            = "0,1,2,5,13,34,89,233,610,1597,4181";
extern bool       AutoMartingale                 = false;
                                                             // other martingale sequences, in decreasing "aggressivity" order:
                                                             // aggressive_fibonacci_sequence="1,2,5,13,34,89,233,610,1597,4181"; // =every second term of fibonacci sequence;
                                                             // powers_of_2_sequence="1,2,4,8,16,32,64,128,256,512,1024,2048,4096";
                                                             // fibonacci_sequence="1,2,3,5,8,13,21,34,55,89,144,233,377,610,987,1597"; // F(n)=F(n-1)+F(n-2);
                                                             // mild_fibonacci_sequence="1,2,3,4,6,9,13,19,28,41,60,88,129,189,277,406"; // F(n)=F(n-1)+F(n-3);
extern bool       RestartMartSeq                 = false;    // If Max Level is exceeded , if false, keep last multiplier till profitable else revert to Mart_Idx=0
extern int        DisplayGapSize                 = 10;
                                                             // Variables that can be imported from the IndicatorV8.1:
extern bool       ImportFromIndicator            = false;    // set this false to allow setting session parameters here rather than importing them from Global Variables set by the Indicator:
extern string     StartTime                      = "00:00";  //"04:00";    // time for start of price establishment window
extern string     EndTime                        = "08:00";  //"07:00";    // time for end of price establishment window
extern string     SessionEndTime                 = "18:00";  //"18:00";   // end of daily session; tomorrow is another day!
extern int        MaxBoxSizeInPips               = 150;      //50;  // box size limit to show Red "NO TRADE" box
extern int        MinBoxSizeInPips               = 5;        // min tradable box size; when box is smaller than that, you should at least reduce your usual lot-size if you decide to trade it;
extern double     EntryFactor                    = 1.0;      //0.3;
extern double     TPFactor                       = 2.58;     // if >=10, this will be taken as FIXED PIPs rather than a factor of the box size;
extern int        SLpips                         = 0;        // if non-0, then SLFactor is ignored;
extern double     SLFactor                       = 1.000;
extern double     LevelsResizeFactor             = 1.0;
extern int        MinExtentInPips                = 0;        // actual box extent min limit for computing trade levels;
extern int        MaxExtentInPips                = 0;        // actual box extent max limit for computing trade levels; 0 will default to MaxBoxSizeInPips;
extern int        Slippage                       = 2;        // slippage in pips; also used to check if we can open a market order when price is too close for pending orders;

extern bool       ShowBoxes                      = true;     // when true, will show boxes and fibs just like the Indicator would do; this avoids using the indicator AND the EA together;
extern bool       LimitBoxToMaxSize              = true;     // when true, a box larger than MaxBoxSizeInPips will be limited to MaxBoxSizeInPips.
extern color      BoxColorOK                     = LightBlue;
extern color      BoxColorNOK                    = Red;
extern color      BoxColorMAX                    = Orange;
extern color      SessionColor                   = C'255,238,210'; // show Session periods with a different background color
extern color      LevelColor                     = Linen; 

//
//
//
//
//

#define  NL                  "\n"
#define  Status_initializing "????? I N I T I A L I Z I N G ?????"
#define  Status_initialized  "..... I N I T I A L I Z E D ....."
#define  Status_trading      "+++++ T R A D I N G +++++"
#define  Status_not_trading  "!!!!! N O T   T R A D I N G (Box is too large) !!!!!"
#define  Status_finished     "----- F I N I S H E D   F O R   T O D A Y -----"

  double  LongPendingEntryPrice;
  double  LongTP;
  double  LongSL;
  double  ShortPendingEntryPrice;
  double  ShortTP;
  double  ShortSL;
  double  vSellStop;
  double  vBuyStop;
  double  _EntryFactor;
  double  _TPFactor;
  double  _LevelsResizeFactor;
  double  boxHigh;
  double  boxLow;
  double  boxExtent;
  double  boxMedianPrice;
  double  LondonOpens;
  double  LondonCloses;
  double  pip;
  double  minlot;
  double  MartFactor[];

  datetime tBoxStart;
  datetime tBoxEnd;
  datetime tSessionStart;
  datetime tSessionEnd;

  int SL_pips,TP_pips;
  int iStartTime,iEndTime,iSessionEndTime;
  int _iStartTime,_iEndTime,_iSessionEndTime;
  int _MaxBoxSizeInPips;  // box size limit to show Red "NO TRADE" box
  int _MinExtentInPips;   // actual box extent min limit for computing trade levels;
  int _MaxExtentInPips;   // actual box extent max limit for computing trade levels; 0 will default to MaxBoxSizeInPips;
  int StartShift;
  int EndShift;
  int LastOrderProfitable=false;
  int TicketNo;
  int MartDepth=0;
  int Mart_Idx=0;
  int digits;
  int maxSL;
  int TradesOpen, TradesWon, TradesLost, TradesTotal;
  
  string symbol;
  string TradingStatus;
  string comment, Gap;
  string DisabledMessage;
  string op_str[5] = { "BUY","SELL","BUYLIMIT","SELLLIMIT","BUYSTOP","SELLSTOP"};

  bool   TradeHour = false, prev_TradeHour=false;
  bool   ClosedForFriday;
  bool   ClosedForSaturday;
  bool   TradeOpen;
  bool   UseMartingale=false;
  bool   RobotDisabled;
 
//
//
//
//
//

int init()
{

   RobotDisabled = false;
   TradingStatus = Status_initializing;

   getpip();

   symbol = Symbol(); 

   if (MagicNumber == 0) { MagicNumber = create_MagicNumber("");  }

   Slippage = Slippage*pip/Point;
   
   //
   //
   //
   //
   //
   
   if (!IsDemo())
   {
     if (MagicNumber == 8682371)
     {
       MessageBox("You cannot run this robot until changed the MagicNumber input from the default of 8682371." + NL 
                + "Please reload the robot with a new MagicNumber input.");
       DisabledMessage = "You need to change the MagicNumber input";
       RobotDisabled = true;
       _Alert (DisabledMessage);
       Comment("The robot has been suspended. Reason: ", DisabledMessage);
       return;
     }
  
     if (MagicNumber < 10000) 
     { 
        MessageBox("Your MagicNumber input needs to be at least 5 Digits long, your MagicNumber is " + MagicNumber + NL 
                  + "Please reload the robot with the MagicNumber input containing at least 5 Digits - the more the better.");
        DisabledMessage = "You need to make the MagicNumber input at 5 Digits long - more is better";
        _Alert (DisabledMessage);
        Comment("The robot has been suspended. Reason: ", DisabledMessage);
        RobotDisabled = true;
        return;
      }
   }
   
   //
   //
   //
   //
   //
   
   if (Lot < minlot) 
   {
      DisabledMessage = "Lot must be >= "+minlot+NL;
      _Alert (DisabledMessage);
      Comment("The robot has been suspended. Reason: ", DisabledMessage);
      RobotDisabled = true;
      return;
   }

   if (AllowHalfClose && Lot/2 < minlot) 
   {
      DisabledMessage = "Lot must be >= "+2*minlot+ " to allow HalfClose option;"+NL;
      _Alert (DisabledMessage);
      Comment("The robot has been suspended. Reason: ", DisabledMessage);
      RobotDisabled = true;
      return;
   }

   //Check MaxRisk value
   //
   //
   //
   //
   //
   
   if (MaxRisk > 100) 
   {
      MessageBox("INVALID MaxRisk value " + MaxRisk + NL 
                 + "Please reload the robot with valid MaxRisk.");
      DisabledMessage = "INVALID MaxRisk value " + MaxRisk;
      RobotDisabled = true;
      _Alert (DisabledMessage);
      Comment("The robot has been suspended. Reason: ", DisabledMessage);
      return;
    }
    
    //
    //
    //
    //
    //
    
    Gap=""; for (int cc=0; cc< DisplayGapSize; cc++) Gap = StringConcatenate(Gap, " ");

    if (ImportFromIndicator) 
    {
      if (!import_LB_Indi_SETTINGS(objPrefix,symbol)) 
      {
        DisabledMessage = "Some Indicator SETTINGS are missing;\n    Please verify objPrefix in the Indicator and the EA;\n    (objPrefix is currently set to "+objPrefix+" in the EA)";
        RobotDisabled = true;
        _Alert (DisabledMessage);
        Comment("The robot has been suspended. Reason: ", DisabledMessage);
        return;
      }

      update_local_LB_Indi_SETTINGS();
    }
  
    // compute LEVELS values:
    //
    //
    //
    //
    
    compute_LB_Indi_LEVELS(Time[0]);
  
    //Check MartFactor and MartDepth for "Martigaling"
    //
    //
    //
    //
    
    if (AutoMartingale) 
    {
      UseMartingale = true;
    } 
    else 
    {
      MartDepth = string_list_to_double_array(martingale_sequence, ",", MartFactor);
      if (MartFactor[0] != 0) 
      {
        UseMartingale = true;
      }
    }
  
    //Must not use OneCancelsTheOther when Martingale is used
    //
    //
    //
    //
    
    if (!OneCancelsTheOther && UseMartingale)
    {
      MessageBox("INVALID OneCancelsTheOther and UseMartingale values."  
                 + "Please change OneCancelsTheOther to TRUE  for Martingale ");
      DisabledMessage = "INVALID OneCancelsTheOther and UseMartingale values";
      RobotDisabled = true;
      _Alert (DisabledMessage);
      Comment("The robot has been suspended. Reason: ", DisabledMessage);
      return;
    }
  
    TradingStatus = Status_initialized;
    DisplayUserFeedback();
    _Alert(TradingStatus);
return(0);
}
  
//
//
//
//
//

int deinit()
{

   Comment("");
   if (!IsTesting()) RemoveObjects(objPrefix);	
   if (ClosePendingWhenUnload) CloseAllPendingTrades();

return(0);
}

//
//
//
//
//

int start()
{
   double servers_min_stop = MarketInfo(symbol, MODE_STOPLEVEL) * MarketInfo(symbol, MODE_POINT);

   if (RobotDisabled)
   {
      Comment("The robot has been suspended. Reason: ", DisabledMessage);
      return;
   }

   if (ImportFromIndicator) 
   {
     if (!import_LB_Indi_SETTINGS(objPrefix,symbol)) 
     {
        DisabledMessage = "Some Indicator SETTINGS are missing;\n    Please verify objPrefix in the Indicator and the EA;\n    (objPrefix is currently set to "+objPrefix+" in the EA)";
        RobotDisabled = true;
        _Alert (DisabledMessage);
        return;
     }

     // if Indicator SETTINGS have been changed, then CLOSE ALL TRADES (pending AND open trades);
     // EA will reopen pending trades based on the new settings;
     //
     //
     //
     
     if (update_local_LB_Indi_SETTINGS()) { CloseAllPendingTrades(); }
   }
  
   // compute LEVELS values:
   //
   //
   //
   //
   
   compute_LB_Indi_LEVELS(Time[0]);
   if (ShowBoxes == true) { show_boxes(TimeCurrent()); }

   if (!IsTradeAllowed())
   {
    Comment("Trading disabled by the criminal pillock. Please wait for him to get out of bed.");
    return;
   }
   
   //Trading hours
   //
   //
   //
   //
   
   int now = TimeHour(Time[0])*100 + TimeMinute(Time[0]);
   prev_TradeHour = TradeHour;
   if ((LondonOpens<=LondonCloses && (now>=LondonOpens && now<LondonCloses)) || (LondonOpens>LondonCloses && (now>=LondonOpens || now<LondonCloses))) 
   {       TradeHour = true;  } 
   else  { TradeHour = false; }
   
   //
   //
   //
   //
   //
   
   if (prev_TradeHour==false && TradeHour==true) 
   {
     //new session starting now: display new levels
     Print ("New Session Starting: "+ TimeToStr(TimeLocal())+NL
            +"Long Entry: "+ DoubleToStr(LongPendingEntryPrice, Digits)+ ", TP: "+ DoubleToStr(LongTP, Digits)+ ", SL: "+ DoubleToStr(LongSL, Digits)+ NL
            +"Short Entry: "+ DoubleToStr(ShortPendingEntryPrice, Digits)+ ", TP: "+ DoubleToStr(ShortTP, Digits)+ ", SL: "+ DoubleToStr(ShortSL, Digits)+ NL);
   }

   if (!TradeHour)
   {
     if (TradingStatus != Status_finished) 
     {
        _Alert (Status_finished+" Session End at "+DoubleToStr(LondonCloses/100,2));
        CloseAllPendingTrades();
     }
        TradingStatus = Status_finished;
        DisplayUserFeedback();
     return;
    }
    
    //
    //
    //
    //
    //

    // if box is too large, don't trade today!
    GetTradingStatus();
    if (TradingStatus == Status_not_trading)
    {
      DisplayUserFeedback();
      return;
    }
    
    //
    //
    //
    //
    // Count total/won/lost/open trades
    
    GetAggregatePosition();            

    if (OrdersTotal() > 0 && UseSelectedManagementFunctions) { ManageOpenTrades();  }
    
    //
    //
    //
    //
    //Max trades filter
    
    if (MaxTradesAllowedForTheDay > 0)
    {                
      if (TradesTotal >= MaxTradesAllowedForTheDay) 
      {
        if (TradingStatus != Status_finished) 
        {
          _Alert (Status_finished+"Reached Max Trades at "+TimeToStr(TimeCurrent(),TIME_SECONDS));
          CloseAllPendingTrades();
        }
        TradingStatus = Status_finished;
        DisplayUserFeedback();
        return;
      }
    }
    
    //
    //
    //
    //
    //Max loss filter
    
    if (MaxLossesAllowedPerPair > 0)
    {
      if (TradesLost >= MaxLossesAllowedPerPair) 
      {
        if (TradingStatus != Status_finished) 
        {
          _Alert (Status_finished+"Reached MAX LOSSES at "+TimeToStr(TimeCurrent(),TIME_SECONDS));
          CloseAllPendingTrades();
        }
        TradingStatus = Status_finished;
        DisplayUserFeedback();
        return;
      }
    }
    
    //
    //
    //
    //
    //


    if (IsTradeAllowed())
    {                    
      TradingStatus = Status_trading;

      RefreshRates();
    
      //If market is in the box, then check to see if pending trades need sending, and send them if so
      //Long trade
      //
      //
      //
      
      double lotsToTrade=0;
      if (!DoesTradeExist(OP_BUYSTOP) && !DoesTradeExist(OP_BUY) && !DoesTradeExist(OP_SELL)) 
      {
        // no pending or open order yet: try to open a pending order
        if (Ask <= LongPendingEntryPrice-servers_min_stop) 
        {
          lotsToTrade = get_lotsToTrade(TP_pips,SL_pips);
          OpenTrade(OP_BUYSTOP, lotsToTrade, LongPendingEntryPrice, LongSL, LongTP);
        } // CANNOT open pending BuyStop order: price is too close to minimum server stop level: check is we can open a market order instead
        else if (Ask >= LongPendingEntryPrice && Ask <= LongPendingEntryPrice+Slippage*Point) 
        {
          lotsToTrade = get_lotsToTrade(TP_pips,SL_pips);
          OpenTrade(OP_BUY, lotsToTrade, Ask, LongSL, LongTP);
        }
      }
      
      //Short
      //
      //
      //
      //
      
      if (!DoesTradeExist(OP_SELLSTOP) && !DoesTradeExist(OP_SELL) && !DoesTradeExist(OP_BUY)) 
      {
        // no pending or open order yet: try to open a pending order
        if (Bid >= ShortPendingEntryPrice+servers_min_stop) 
        {
          if (lotsToTrade==0) lotsToTrade = get_lotsToTrade(TP_pips,SL_pips);// use same lots as BUYSTOP order
            OpenTrade(OP_SELLSTOP, lotsToTrade, ShortPendingEntryPrice, ShortSL, ShortTP);
        } // CANNOT open pending SellStop order: price is too close to minimum server stop level: check is we can open a market order instead
        else if (Bid <= ShortPendingEntryPrice && Bid >= ShortPendingEntryPrice-Slippage*Point) 
        {
          if (lotsToTrade==0) lotsToTrade = get_lotsToTrade(TP_pips,SL_pips);// use same lots as BUYSTOP order
            OpenTrade(OP_SELL, lotsToTrade, Bid, ShortSL, ShortTP);
        }
      }
    }
    
    //
    //
    //
    //
    //
        
    DisplayUserFeedback();
return(0);
}

//
//
//
//
//

double get_lotsToTrade(int TP_pips, int SL_pips)
{
      double lotsToTrade,last_profit=0,total_loss=0,factor=1;
      int last_ticket=0;
      
      //
      //
      //
      //
      //
      
      if (!UseMartingale)
      {
        //adjust Lot based on MaxRisk and SL value
        if (MaxRisk > 0) return (MathMin(Lot,GetMaxLot(SL_pips,MaxRisk)));
        else return (Lot);
      }

      // Martingale is used: determine the martingale factor based result of last trade:

      Mart_Idx=0;

      //Search trade history for past trades, start from end of history(=newest trades)
      //if no past buy/sell trade can be found, then the mart factor will not change;
      //
      //
      //
      
      for (int cc = OrdersHistoryTotal()-1; cc >= 0; cc--)
      {      
         OrderSelect(cc, SELECT_BY_POS, MODE_HISTORY);
         if (OrderMagicNumber() == MagicNumber && OrderSymbol() == symbol)       
         {
           if (OrderType()==OP_BUY || OrderType()==OP_SELL) 
           {
             if (last_ticket==0) 
             {
               last_ticket = OrderTicket();
               last_profit = OrderProfit();
             }

             if (OrderProfit() > 0 ) break; // first win: factor is now set

             if (AutoMartingale) // auto-martingale: recoup past losses, and add current TP
             { 
               total_loss = total_loss + MathAbs(OrderClosePrice()-OrderOpenPrice())*OrderLots()/Lot;
               factor = 1 + total_loss/pip / TP_pips;
             }  
             else 
             { // use fixed Martingale sequence: move to next mart factor
                Mart_Idx++;
                // check if we reached the top mart factor
                if (Mart_Idx > MartDepth-1) 
                {
                  if(RestartMartSeq) Mart_Idx = 0;
                  else Mart_Idx = MartDepth-1;
                }
                factor = MartFactor[Mart_Idx];
             }
          }
       }
    }

    if (AutoMartingale) { Print ("### AutoMartingale: ",total_loss/pip," pips to recoup, +",TP_pips," pips TP => MartFactor is ", factor);} 
    else                { Print ("###",last_ticket," profit=",last_profit," using MartFactor[",Mart_Idx,"]=",factor);}
  
return (Lot*factor);
}

//
//
//
//
//

double GetMaxLot(int SL, double Risk)
{
      double pipvaluefor1lot,maxLot;
      
      //
      //
      //
      //
      //
      
      pipvaluefor1lot = MarketInfo(Symbol(),MODE_TICKVALUE)*Point/MarketInfo(Symbol(),MODE_TICKSIZE) * pip;
      maxLot    = NormalizeDouble(AccountFreeMargin()*Risk/100/pipvaluefor1lot/SL,2);
      
return(maxLot);
}

//
//
//
//
//

bool DoesTradeExist(int type)
{
    //Searches for open trades
    for (int cc = OrdersTotal() - 1; cc >= 0; cc--)
    {      
       if (!OrderSelect(cc, SELECT_BY_POS)) continue;
       if (OrderMagicNumber() == MagicNumber && OrderSymbol() == symbol && OrderCloseTime() == 0) 
       {
         if (OrderType() == type) return(true);
       }
    }  
return(false); 
} 

//
//
//
//
//

void OpenTrade(int type, double lots, double price, double SL, double TP) //OP_BUYSTOP or OP_SELLSTOP
{
    _Alert("Sending "+op_str[type]+" at "+price+" "+lots+" lots");
    _OrderSend(symbol, type, lots, price, Slippage, SL, TP, TradeComment, MagicNumber, 0,Blue);

}

//
//
//
//
//

void GetAggregatePosition()
{
    //returns the total trades for the pair - TradesOpen, TradesWon, TradesLost
   
    TradesOpen  = 0;
    TradesWon   = 0;
    TradesLost  = 0;
    TradesTotal = 0;

    //Count Open trades
    //
    //
    //
    //
    
    if (OrdersTotal() > 0)
    {
      for (int cc = OrdersTotal() - 1; cc >= 0; cc--)
      {      
         if (!OrderSelect(cc, SELECT_BY_POS, MODE_TRADES) ) continue;
         if (OrderMagicNumber() == MagicNumber && OrderSymbol() == symbol && OrderCloseTime() == 0)
           { TradesOpen++; }
      }
    }
   
   //Count won/lost trades in the past 24 hours from trades History
   //
   //
   //
   //
   
   for (cc = OrdersHistoryTotal() - 1; cc >= 0; cc--)
   {      
      OrderSelect(cc, SELECT_BY_POS, MODE_HISTORY);
      if (OrderMagicNumber() == MagicNumber && OrderSymbol() == symbol)       
      {
        if (OrderCloseTime()<tBoxEnd) break;// past 24 hours, stop counting;
        if (OrderProfit() < 0) TradesLost++;
        if (OrderProfit() > 0) TradesWon++;         
      }
   }
   
   TradesTotal = TradesOpen + TradesLost + TradesWon;

}

//
//
//
//
//

void GetTradingStatus()
{
   
    if (MaxBoxSizeInPips > 0 && boxExtent > MaxBoxSizeInPips * pip) TradingStatus = Status_not_trading;

}

//
//
//
//
//

void ManageOpenTrades()
{
    //Called if there are any open trades
    //
    //
    //
    //
    
    for (int cc = OrdersTotal() - 1; cc >=0; cc--)
    {
       if (!OrderSelect(cc, SELECT_BY_POS)) continue;

        if (OrderMagicNumber() ==  MagicNumber && OrderSymbol() == symbol && OrderCloseTime() == 0)
        {
          TicketNo = OrderTicket();

          // Half close at half-TP ?
          //
          //
          //
          //
          
          if (AllowHalfClose) 
          {
            if (OrderCloseTime() == 0 && OrderProfit() > 0 && OrderLots() == Lot) 
            {
              TicketNo = ShouldHalfTradeClose(TicketNo); // returns new ticket assigned to the half-closed trade;
            }
          }

          // Move to BreakEven+some_pips after some profit ?
          //
          //
          //
          //
          
          if (BreakEvenPips > 0) {  BreakEvenStopLoss(BreakEvenPips,BreakEvenProfitInPips); }
   
          // OCO ?
          //
          //
          //
          //
          
          if (OneCancelsTheOther) { OCO(); }

          // Trailing stop ?
          //
          //
          //
          //
          
          if (TrailingStopPips > 0) { Trail_order(TrailingStopPips,TrailingStopStep); }

       }
   }
}

//
//
//
//
//

int ShouldHalfTradeClose(int TicketNo)
{
   //Only called if selected trade is in profit;
   //Closes half the trade at profit of TP / 2;
   
   bool result;
   double SL;
   double HcPrice = NormalizeDouble((OrderTakeProfit() + OrderOpenPrice()) / 2, Digits);//Half close price
   
   //Long
   //
   //
   //
   //
   
   if (OrderType() == OP_BUY && Bid > HcPrice) 
   {
     _Alert(" #"+OrderTicket()+" half-close Buy at "+Bid);
     result = _OrderClose(OrderTicket(),OrderLots() / 2, Bid,5*pip/Point,Red);
     if (result==false) return (TicketNo);
     Sleep(100);
     result = DoesTradeExist(OP_BUY);//Re-selects the half-closed trade
     if (result==false) 
     {
       _Alert(" #"+OrderTicket()+" DoesTradeExist(OP_BUY) FAILED ");
       return (TicketNo);
     }
     TicketNo = OrderTicket();//get new ticket, will be returned to caller;
     if (OrderStopLoss() < OrderOpenPrice())
     {
       SL = NormalizeDouble(OrderOpenPrice() + (BreakEvenProfitInPips * pip), Digits);
       _Alert(" #"+OrderTicket()+" moving to BE+"+BreakEvenProfitInPips+"pips");
       result = _OrderModify(OrderTicket(), OrderOpenPrice(), SL, OrderTakeProfit(), OrderExpiration(), Green);
     }
    return (TicketNo);
  }
   
  //Short
  //
  //
  //
  //
  
  if (OrderType() == OP_SELL && Ask < HcPrice) 
  {
    _Alert(" #"+OrderTicket()+ " half-close Sell at "+ Ask);
    result = _OrderClose(OrderTicket(),OrderLots() / 2, Ask,5*pip/Point,Red);
    if (result==false) return (TicketNo);
    Sleep(100);
    result = DoesTradeExist(OP_SELL);//Re-selects the half-closed trade
    if (result==false) 
    {
      _Alert(" #"+OrderTicket()+ " DoesTradeExist(OP_SELL) FAILED ");
      return (TicketNo);
    }
    TicketNo = OrderTicket();//get new ticket, will be returned to caller;
    if (OrderStopLoss() > OrderOpenPrice() || OrderStopLoss()==0)
    {
      SL = NormalizeDouble(OrderOpenPrice() - (BreakEvenProfitInPips * pip), Digits);
      _Alert(" #"+OrderTicket()+ " moving to BE+"+BreakEvenProfitInPips+"pips");
      result = _OrderModify(OrderTicket(), OrderOpenPrice(), SL, OrderTakeProfit(), OrderExpiration(), Green);
    }
    return (TicketNo);
  }   
return (TicketNo);
}

//
//
//
//
//

void BreakEvenStopLoss(int profit_pips_target, int pips_to_lock)
{
    //Lock-in some profit after a minimum profit target is reached:
    //(move SL to BE+pips_to_lock)
    double SL;
   
    //Long
    //
    //
    //
    //
    
    if (OrderType() == OP_BUY && Bid >= OrderOpenPrice()+profit_pips_target*pip && OrderStopLoss() < OrderOpenPrice())
    {
      SL = NormalizeDouble(OrderOpenPrice() + (pips_to_lock*pip), Digits);
      _Alert(" #"+OrderTicket()+" moving to BE+"+pips_to_lock+"pips");
      _OrderModify(OrderTicket(), OrderOpenPrice(), SL, OrderTakeProfit(), OrderExpiration(), Green);
    }
   
    //Short
    //
    //
    //
    //
    
    if (OrderType() == OP_SELL && Ask <= OrderOpenPrice()-profit_pips_target*pip && (OrderStopLoss() > OrderOpenPrice() || OrderStopLoss()==0)) 
    {
      SL = NormalizeDouble(OrderOpenPrice() - (pips_to_lock*pip), Digits);
      _Alert(" #"+OrderTicket()+" moving to BE+"+pips_to_lock+"pips");
      _OrderModify(OrderTicket(), OrderOpenPrice(), SL, OrderTakeProfit(), OrderExpiration(), Green);
    }
   
}

//
//
//
//
//

void OCO() 
{
    //Deletes a pending trade when its partner has filled
    if (OrderType()==OP_BUY) 
    {
      if (DoesTradeExist(OP_SELLSTOP)) 
      {
        _Alert(" #"+OrderTicket()+ " OCO(): deleting pending SELLSTOP");
        _OrderDelete(OrderTicket(),CLR_NONE);
      }
    }
    
    //
    //
    //
    //
    //
    
    if (OrderType()==OP_SELL) 
    {
      if (DoesTradeExist(OP_BUYSTOP)) 
      {
        _Alert(" #"+OrderTicket()+ " OCO(): deleting pending BUYSTOP");
        _OrderDelete(OrderTicket(),CLR_NONE);
      }
    }
}

//
//
//
//
//

void Trail_order(int _TrailingStopPips, int _TrailingStopStep)
{
    if (_TrailingStopPips<=0) return;
      if (_TrailingStopStep>_TrailingStopPips) return;
        if (MathAbs(OrderStopLoss()-OrderClosePrice()) > (_TrailingStopPips+_TrailingStopStep)*pip) 
        {										    
          if (OrderType() == OP_BUY)  _OrderModify(OrderTicket(),OrderOpenPrice(),OrderClosePrice()-_TrailingStopPips*pip,OrderTakeProfit(),OrderExpiration(),Purple);
          if (OrderType() == OP_SELL) _OrderModify(OrderTicket(),OrderOpenPrice(),OrderClosePrice()+_TrailingStopPips*pip,OrderTakeProfit(),OrderExpiration(),Purple);
        }
}

//
//
//
//
//

void CloseAllPendingTrades()
{
    bool result,alerted=false;
    int err;
    
    //
    //
    //
    //
    //
    
    for (int cc = OrdersTotal() - 1; cc >=0; cc--)
    {
      if (!OrderSelect(cc, SELECT_BY_POS) ) continue;

       if (OrderMagicNumber() == MagicNumber && OrderSymbol() == symbol && OrderCloseTime() == 0) 
       {

         if (!alerted) 
         {
           alerted=true;
           _Alert(" CLOSING ALL PENDING ORDERS NOW !!!");
         }

         switch (OrderType()) 
         {
           case OP_BUYSTOP:  
           case OP_BUYLIMIT:  
           case OP_SELLSTOP:
           case OP_SELLLIMIT:
           result = _OrderDelete(OrderTicket(),CLR_NONE); break;
           break;
         }
      }
   }
}

//
//
//
//
//

#define MAX_RETRIES 10

int _OrderSend(string symbol, int type, double lots, double price, int slippage, double stoploss, double takeprofit, string comment, int magic, datetime expiration, color arrow_color)
{
   //"reliable" version of OrderSend(): will retry 10 times before failing
   int i,j,err,ticket;
   
   //
   //
   //
   //
   //
   
   for (i=0;i<MAX_RETRIES;i++) 
   {
      for (j=0;j<MAX_RETRIES && !IsTradeAllowed();j++) Sleep(500); // wait for Trade to be allowed(free context)
      if (j==MAX_RETRIES) { _Alert("OrderSend() FAILED: trade not allowed"); return (-1); }

      // send order with 0 SL, then modify order to apply SL -- a common counter-ECN method found in SteveHopwood's bots...
      ticket = OrderSend(symbol, type, lots, price, slippage, 0/*stoploss*/, 0/*takeprofit*/, comment, magic, expiration, arrow_color);
      if (ticket != -1) 
      {
        if (stoploss!=0 || takeprofit!=0) _OrderModify(ticket, price, stoploss, takeprofit, expiration, arrow_color);
        return(ticket);//success
      }
    }

    err = GetLastError();
    _Alert("OrderSend() FAILED at "+DoubleToStr(price,digits)+",SL="+DoubleToStr(stoploss,digits)+",TP="+DoubleToStr(takeprofit,digits)+",Ask="+DoubleToStr(Ask,digits)+" ("+err+"): "+ErrorDescription(err));
return(-1);
}

//
//
//
//
//

bool _OrderModify(int ticket, double price, double stoploss, double takeprofit, datetime expiration, color arrow_color) 
{
    //"reliable" version of OrderModify(): will retry 10 times before failing
    int i,j,err,ret;
    
    //
    //
    //
    //
    //
    
    for (i=0;i<MAX_RETRIES;i++) 
    {
       for (j=0;j<MAX_RETRIES && !IsTradeAllowed();j++) Sleep(500); // wait for Trade to be allowed(free context)
       if (j==MAX_RETRIES) { _Alert("OrderModify(#"+ticket+") FAILED: trade not allowed"); return(false); }

         ret = OrderModify(ticket, price, stoploss, takeprofit, expiration, arrow_color); 
         if (ret == true) return(true);//success
    }

    err = GetLastError();
    _Alert("OrderModify(#"+ticket+") FAILED "+"SL="+DoubleToStr(stoploss,digits)+",TP="+DoubleToStr(takeprofit,digits)+",Ask="+DoubleToStr(Ask,digits)+" ("+err+"): "+ErrorDescription(err));
return(false);
}

//
//
//
//
//

bool _OrderClose( int ticket, double lots, double price, int slippage, color Color) 
{
    //"reliable" version of OrderClose(): will retry 10 times before failing
    int i,j,err;
    bool ret;
    
    //
    //
    //
    //
    //
    
    for (i=0;i<MAX_RETRIES;i++) 
    {
       for (j=0;j<MAX_RETRIES && !IsTradeAllowed();j++) Sleep(500); // wait for Trade to be allowed(free context)
       if (j==MAX_RETRIES) {_Alert("OrderClose(#"+ticket+") FAILED: trade not allowed");return(false);}

       ret = OrderClose(ticket, lots, price, slippage, Color);
       if (ret == true) return(true);//success
    }

    err = GetLastError();
    _Alert("OrderClose(#"+ticket+") FAILED ("+err+"): "+ErrorDescription(err));
return(false);
}

//
//
//
//
//

bool _OrderDelete(int ticket, color Color) 
{
    //"reliable" version of OrderDelete(): will retry 10 times before failing
    int i,j,err;
    bool ret;
    
    //
    //
    //
    //
    //
    
    for (i=0;i<MAX_RETRIES;i++) 
    {
       for (j=0;j<MAX_RETRIES && !IsTradeAllowed();j++) Sleep(500); // wait for Trade to be allowed(free context)
       if (j==MAX_RETRIES) { _Alert("OrderDelete(#"+ticket+") FAILED: trade not allowed"); return(false); }

       ret = OrderDelete(ticket, Color);
       if (ret == true) return(true);//success
    }

    err = GetLastError();
    _Alert("OrderDelete(#"+ticket+") FAILED ("+err+"): "+ErrorDescription(err));
return(false);
}

//
//
//
//
//

int create_MagicNumber(string s)
{
   // create a magic number that is "unique" for a given {EA_name,Symbol,Period} combo
   int magic=0;
   
   //
   //
   //
   //
   //
   
   s = s+WindowExpertName()+Symbol()+Period()+objPrefix;
   magic = hash_string(s);
   while (magic < 10000) 
   {
    s = s+magic;
    magic = hash_string(s);
   }
return (magic);
}

//
//
//
//
//

int hash_string(string s)
{
   // this is the djb2 string hash algo
   int h = 5381, l = StringLen(s), i;
   for (i=0; i<l; i++) h = h * 33 + StringGetChar(s,i);
   return (h);
} 

//
//
//
//
//

void getpip()
{
   if(Digits==2 || Digits==4) pip = Point;
   else if(Digits==3 || Digits==5) pip = 10*Point;
   else if(Digits==6) pip = 100*Point;
      
	if (Digits == 3 || Digits == 2) digits = 2;
	else digits = 4;
	
	minlot = MarketInfo(Symbol(),MODE_MINLOT);
	
} 

//
//
//
//
//

int time_string_to_int(string ts)
{
   int t;
   t = StrToTime("1970.01.01 " + ts)/60;
   return (MathRound(t/60)*100+(t%60));
}

//
//
//
//
//

string time_int_to_string(int t)
{
      if (t>9999) return (StringSubstr(TimeToStr(t),11,5)); // time value includes DATE -- compatible with Indicator < V9
return ((t/100)+":"+(t%100));                               // time is in hhmm integer format (e.g 1400 for "14:00")
}

//
//
//
//
//

int roundup_hhmm_time_to_TF (int hhmm, int tf)
{
   double t  = hhmm; // convert to double for divisions
   double hh = MathFloor(t/100);
   int mm = (t/100-hh)*100;
   mm = (MathFloor(mm/tf)+1)*tf;
return ( (hh+MathFloor(mm/60))*100 + mm-60*MathFloor(mm/60));
}

//
//
//
//
//

void _Alert(string text)
{
    Alert ("LB EA: ", symbol, " ", text);
}

//
//
//
//
//

bool import_LB_Indi_SETTINGS(string objPrefix, string symbol)
{
    bool ret=true;
    
    //
    //
    //
    //
    //
    
    if (!GlobalVariableCheck(objPrefix+symbol+"-StartTime")) return(false);
    _iStartTime       = GlobalVariableGet(objPrefix+symbol+"-StartTime");
  
    if (!GlobalVariableCheck(objPrefix+symbol+"-EndTime")) return(false);
    _iEndTime         = GlobalVariableGet(objPrefix+symbol+"-EndTime");

    if (!GlobalVariableCheck(objPrefix+symbol+"-SessionEndTime")) return(false);
    _iSessionEndTime  = GlobalVariableGet(objPrefix+symbol+"-SessionEndTime");

    if (!GlobalVariableCheck(objPrefix+symbol+"-MaxBoxSizeInPips")) return(false);
    _MaxBoxSizeInPips = GlobalVariableGet(objPrefix+symbol+"-MaxBoxSizeInPips");

    if (!GlobalVariableCheck(objPrefix+symbol+"-EntryFactor")) return(false);
    _EntryFactor      = GlobalVariableGet(objPrefix+symbol+"-EntryFactor");

    if (!GlobalVariableCheck(objPrefix+symbol+"-TPFactor")) return(false);
    _TPFactor        = GlobalVariableGet(objPrefix+symbol+"-TPFactor");

    if (!GlobalVariableCheck(objPrefix+symbol+"-LevelsResizeFactor")) return(false);
    _LevelsResizeFactor = GlobalVariableGet(objPrefix+symbol+"-LevelsResizeFactor");

    if (!GlobalVariableCheck(objPrefix+symbol+"-MinExtentInPips")) return(false);
    _MinExtentInPips = GlobalVariableGet(objPrefix+symbol+"-MinExtentInPips");

    if (!GlobalVariableCheck(objPrefix+symbol+"-MaxExtentInPips")) return(false);
    _MaxExtentInPips = GlobalVariableGet(objPrefix+symbol+"-MaxExtentInPips");

return (ret);
}

//
//
//
//
//

bool update_local_LB_Indi_SETTINGS()
{
    bool ret=false;
    
    //
    //
    //
    //
    //

    if (_iStartTime != iStartTime) 
    {
      iStartTime = _iStartTime;
      StartTime  = time_int_to_string(iStartTime);
      ret=true;
    }
    if (_iEndTime != iEndTime) 
    {
      iEndTime = _iEndTime;
      EndTime  = time_int_to_string(iEndTime);
      ret=true;
    }
    if (_iSessionEndTime != iSessionEndTime) 
    {
      iSessionEndTime = _iSessionEndTime;
      SessionEndTime  = time_int_to_string(iSessionEndTime);
      ret=true;
    }
    if (_MaxBoxSizeInPips != 0 && _MaxBoxSizeInPips != MaxBoxSizeInPips) 
    {
      MaxBoxSizeInPips = _MaxBoxSizeInPips;
      ret=true;
    }
    if (_EntryFactor != 0 && _EntryFactor != EntryFactor) 
    {
      EntryFactor = _EntryFactor;
      ret=true;
    }
    if (_TPFactor != 0 && _TPFactor != TPFactor) 
    {
      TPFactor = _TPFactor;
      ret=true;
    }
    if (_LevelsResizeFactor != 0 && _LevelsResizeFactor != LevelsResizeFactor) 
    {
      LevelsResizeFactor = _LevelsResizeFactor;
      ret=true;
    }
    if (_MinExtentInPips != 0 && _MinExtentInPips != MinExtentInPips) 
    {
      MinExtentInPips = _MinExtentInPips;
      ret=true;
    }
    if (_MaxExtentInPips != 0 && _MaxExtentInPips != MaxExtentInPips) 
    {
      MaxExtentInPips = _MaxExtentInPips;
      ret=true;
    }
  return (ret);
}

//
//
//
//
//

void compute_LB_Indi_LEVELS(datetime thistime)
{
    tBoxStart = StrToTime(TimeToStr(thistime,TIME_DATE) + " "  + StartTime);
    tBoxEnd   = StrToTime(TimeToStr(thistime,TIME_DATE) + " "  + EndTime);
    if (tBoxStart > tBoxEnd) tBoxStart = tBoxStart - 24*60*60; // midnight wrap fix
    if (thistime < tBoxEnd) 
    {
      tBoxStart = tBoxStart - 24*60*60;
      tBoxEnd   = tBoxEnd   - 24*60*60;
      while ((TimeDayOfWeek(tBoxStart)==0 || TimeDayOfWeek(tBoxStart)==6) && (TimeDayOfWeek(tBoxEnd)==0 || TimeDayOfWeek(tBoxEnd)==6)) 
      {
      // box on saturday or sunday: move back 24hours again
        tBoxStart = tBoxStart - 24*60*60;
        tBoxEnd   = tBoxEnd   - 24*60*60;
      }
    }

    tSessionStart = tBoxEnd;
    tSessionEnd = StrToTime(TimeToStr(tSessionStart,TIME_DATE) + " "  + SessionEndTime);
    if (tSessionStart > tSessionEnd) tSessionEnd = tSessionEnd + 24*60*60; // midnight wrap fix

    // setup session open/close times
    LondonOpens  = roundup_hhmm_time_to_TF (time_string_to_int(EndTime), Period());
    LondonCloses = time_string_to_int(SessionEndTime);

    //get hi/lo
    StartShift     = iBarShift(NULL,0,tBoxStart);
    EndShift       = iBarShift(NULL,0,tBoxEnd);
    boxHigh        = High[iHighest(NULL,0,MODE_HIGH,(StartShift-EndShift+1),EndShift)];
    boxLow         = Low[iLowest(NULL,0,MODE_LOW,(StartShift-EndShift+1),EndShift)];
    boxExtent      = boxHigh - boxLow;
    boxMedianPrice = (boxHigh+boxLow)/2;

    // "resize" the Box with LevelsResizeFactor before computing levels
    double extent = boxExtent;
    if (MaxExtentInPips > 0 && boxExtent > MaxExtentInPips*pip) extent = MaxExtentInPips*pip;
    if (MinExtentInPips > 0 && boxExtent < MinExtentInPips*pip) extent = MinExtentInPips*pip;
    extent    = extent * LevelsResizeFactor;
    vSellStop = boxMedianPrice - extent/2;
    vBuyStop  = boxMedianPrice + extent/2;

    // Entry and TP prices
    LongPendingEntryPrice  = NormalizeDouble(vBuyStop*EntryFactor,Digits);
    ShortPendingEntryPrice = NormalizeDouble(vSellStop*EntryFactor,Digits);
    
    
    if (TPFactor < 10) TP_pips = boxExtent*TPFactor/pip;
    else            {  TP_pips = TPFactor; TPFactor = TP_pips*pip/extent; }
    LongTP  = NormalizeDouble(vBuyStop + extent*TPFactor, Digits);
    ShortTP = NormalizeDouble(vSellStop - extent*TPFactor, Digits);

    if (SLpips <= 0) SL_pips = extent*SLFactor/pip; // SL is a factor of the Box size
    else SL_pips = SLpips; // fixed pips SL
    LongSL  = NormalizeDouble(vBuyStop  - SL_pips*pip,Digits);
    ShortSL = NormalizeDouble(vSellStop + SL_pips*pip,Digits);
}

//
//
//
//
//

int string_list_to_double_array(string list, string sep, double &array[])
{

   int i=0,j,k;
   string ls;
   
   //
   //
   //
   //
   //
   
   while (true) 
   {
   	j=StringFind(list,sep,i);
   	if (j<0) break;

 		ls=StringTrimRight(StringTrimLeft(StringSubstr(list,i,j-i)));
 		i=j+1;
 		k++;
 		ArrayResize(array,k);
 		array[k-1]=StrToDouble(ls);
  }
  //last element in the list:
	ls=StringTrimRight(StringTrimLeft(StringSubstr(list,i)));
	k++;
	ArrayResize(array,k);
	array[k-1]=StrToDouble(ls);

  return (ArraySize(array));
}

//
//
//
//
//

void DisplayUserFeedback()
{
   static bool alreadyPrinted=false;
  
   if (IsTesting()&& !IsVisualMode() && alreadyPrinted) return; // saves cpu time when backtesting

   comment = Gap+WindowExpertName()+ " "+ EA_VERSION+ NL;
   if (MaxRisk <= 0) {
     comment = comment+Gap+ "Lot size = "+ Lot+ " (MaxRisk not used)"+ NL;
   } else {
     comment = comment+Gap+ "MaxRisk = "+ MaxRisk+ "% => "+ DoubleToStr(Lot,2)+ "Lots, AccountFreeMargin = "+ DoubleToStr(AccountFreeMargin(),0)+ AccountCurrency()+ NL;
   }
   comment = comment+Gap+ "Magic number = "+ MagicNumber+ ", objPrefix = "+ objPrefix+ NL
                    +Gap+ "Trade comment = "+ TradeComment+ NL
                    +Gap+ "Local time: "+ TimeToStr(TimeLocal())+ " Broker time: "+ TimeToStr(TimeCurrent())+ " Current Bar time: "+ TimeToStr(Time[0])+ NL
                    +Gap+ "Trading Hours: "+ DoubleToStr(LondonOpens/100,2)+ " - "+ DoubleToStr(LondonCloses/100,2)+ NL
                    +Gap+ "Box=["+StartTime+"-"+EndTime+"], Box max "+DoubleToStr(MaxBoxSizeInPips,0)+"p, BoxExtent ["+DoubleToStr(boxLow,Digits)+"-"+DoubleToStr(boxHigh,Digits)+"]="+DoubleToStr(boxExtent/pip,1)+"p"+NL
                    +Gap+ "EntryFactor="+DoubleToStr(EntryFactor,2)+", TPFactor="+DoubleToStr(TPFactor,2)+", LevelsResizeFactor="+DoubleToStr(LevelsResizeFactor,2)+")"+NL
                    +Gap+ "Long Entry: "+ DoubleToStr(LongPendingEntryPrice, Digits)+ ", TP: "+ DoubleToStr(LongTP, Digits)+ ", SL: "+ DoubleToStr(LongSL, Digits)+ NL
                    +Gap+ "Short Entry: "+ DoubleToStr(ShortPendingEntryPrice, Digits)+ ", TP: "+ DoubleToStr(ShortTP, Digits)+ ", SL: "+ DoubleToStr(ShortSL, Digits)+ NL
                    +Gap+ "Slippage="+DoubleToStr(Slippage,0)+NL;

   if (UseSelectedManagementFunctions)
   {
      comment = comment+Gap+ "Using management functions:"+NL;
      if (AllowHalfClose) {
        comment = comment+    Gap+ "     Using half-close, move to BE+"+BreakEvenPips+" pips at half-close"+NL;
      } else {
        comment = comment+    Gap+ "     NOT Using half-close"+NL;
      }
      if (BreakEvenPips>0) {
        comment = comment+Gap+ "     lock-in "+BreakEvenProfitInPips+" pips when reached BE+"+BreakEvenPips+" pips profit"+ NL;
      }
      if (TrailingStopPips>0) {
        comment = comment+Gap+ "     Trailing stops at "+TrailingStopPips+" pips, by steps of "+TrailingStopStep+" pips"+ NL;
      }
      if (OneCancelsTheOther) comment = comment+Gap+ "     OCO ON:   Closing parter pending trade when one fills"+ NL;
      else                    comment = comment+Gap+ "     OCO OFF: NOT Closing parter pending trade when one fills"+ NL;
   }

   if (UseMartingale) {
     if (AutoMartingale) {
        comment =comment+Gap+"     AUTO-MARTINGALE ENABLED"+NL;
     } else {
        comment =comment+Gap+"     MARTINGALE FACTORS: ";
        for (int i=0;i<MartDepth;i++) {
          if (i==Mart_Idx) comment=comment+" ((";
          comment=comment+DoubleToStr(MartFactor[i],0);
          if (i==Mart_Idx) comment=comment+")) ";
          comment=comment+",";
        }
        comment=comment+NL;
     }
   } else {
     comment =comment+Gap+"     Martingale NOT used"+NL;
   }
   
   comment = comment+Gap+ EA_VERSION+ " " + TradingStatus+ NL;

   if (IsTesting() && !IsVisualMode()) {
      Print(comment);
      alreadyPrinted = true;
      return;
   }

   Comment(comment);
}

//
//
//
//
//

void show_boxes(datetime now)
{
  
  bool new_box = drawBoxOnce (objPrefix+"Session-"+TimeToStr(tSessionStart,TIME_DATE | TIME_SECONDS),tSessionStart,0,tSessionEnd,boxHigh,SessionColor,1, STYLE_SOLID, true);

  // draw pre-breakout box blue/red once per Session:
  //
  //
  //
  //
  
  if (new_box == true) 
  {
    string boxName = objPrefix+"Box-"+TimeToStr(now,TIME_DATE)+"-"+StartTime+"-"+EndTime;
    if (boxExtent >= MaxBoxSizeInPips * pip) 
    { 
      if (LimitBoxToMaxSize==false) 
      { 
        drawBox (boxName,tBoxStart,boxLow,tBoxEnd,boxHigh,BoxColorNOK,1, STYLE_SOLID, true);
        DrawLbl(objPrefix+"Lbl-"+TimeToStr(now,TIME_DATE)+"-"+StartTime+"-"+EndTime, "NO TRADE! ("+DoubleToStr(boxExtent/pip,0)+"p)", tBoxStart+(tBoxEnd-tBoxStart)/2,boxLow, 12, "Arial Black",LevelColor,3);
      } 
      else 
      {
        drawBox (boxName,tBoxStart,boxLow,tBoxEnd,boxHigh,BoxColorMAX,1, STYLE_SOLID, true);
        DrawLbl(objPrefix+"Lbl-"+TimeToStr(now,TIME_DATE)+"-"+StartTime+"-"+EndTime, "MAX LIMIT! ("+DoubleToStr(boxExtent/pip,0)+"p)", tBoxStart+(tBoxEnd-tBoxStart)/2,boxLow, 12, "Arial Black",LevelColor,3);
      }
    } else if (boxExtent >= MinBoxSizeInPips * pip) 
      { 
        drawBox (boxName,tBoxStart,boxLow,tBoxEnd,boxHigh,BoxColorOK,1, STYLE_SOLID, true);
        DrawLbl(objPrefix+"Lbl-"+TimeToStr(now,TIME_DATE)+"-"+StartTime+"-"+EndTime, DoubleToStr(boxExtent/pip,0)+"p", tBoxStart+(tBoxEnd-tBoxStart)/2,boxLow, 12, "Arial Black",LevelColor,3);
      } 
      else 
      { // "Caution!" box
        drawBox (boxName,tBoxStart,boxLow,tBoxEnd,boxHigh,BoxColorNOK,1, STYLE_SOLID, true);
        DrawLbl(objPrefix+"Lbl-"+TimeToStr(now,TIME_DATE)+"-"+StartTime+"-"+EndTime, "Caution! ("+DoubleToStr(boxExtent/pip,0)+"p)", tBoxStart+(tBoxEnd-tBoxStart)/2,boxLow, 12, "Arial Black", BoxColorNOK, 3);
    }
  }
}
    
//--------------------------------------------------------------------------------------
// drawBox
//--------------------------------------------------------------------------------------
//
//

void drawBox (string objname,datetime tStart, double vStart, datetime tEnd, double vEnd,color c, int width, int style, bool bg)
{
  if (ObjectFind(objname) == -1) 
  {
    ObjectCreate(objname, OBJ_RECTANGLE, 0,tStart,vStart,tEnd,vEnd);
  } 
  else 
  {
    ObjectSet(objname, OBJPROP_TIME1, tStart);
    ObjectSet(objname, OBJPROP_TIME2, tEnd);
    ObjectSet(objname, OBJPROP_PRICE1,vStart);
    ObjectSet(objname, OBJPROP_PRICE2,vEnd);
  }

    ObjectSet(objname, OBJPROP_COLOR,c);
    ObjectSet(objname, OBJPROP_BACK, bg);
    ObjectSet(objname, OBJPROP_WIDTH,width);
    ObjectSet(objname, OBJPROP_STYLE,style);
} 

//--------------------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------------------
//
//

bool drawBoxOnce (string objname,datetime tStart, double vStart, datetime tEnd,   double vEnd,color c, int width, int style, bool bg)
{
    if (ObjectFind(objname) != -1) return(false);//box already exists
  
    ObjectCreate(objname, OBJ_RECTANGLE,0,tStart,vStart,tEnd,vEnd);
       ObjectSet(objname,OBJPROP_COLOR, c);
       ObjectSet(objname, OBJPROP_BACK, bg);
       ObjectSet(objname, OBJPROP_WIDTH,width);
       ObjectSet(objname, OBJPROP_STYLE,style);
return(true);
} 

//
// 
//
//
//

void DrawLbl(string objname, string s, int LTime, double LPrice, int FSize, string Font, color c, int width)
{
    if (ObjectFind(objname) < 0) 
    {
      ObjectCreate(objname, OBJ_TEXT, 0, LTime, LPrice);
    } 
    else 
    {
    if (ObjectType(objname) == OBJ_TEXT) {
         ObjectSet(objname, OBJPROP_TIME1, LTime);
         ObjectSet(objname, OBJPROP_PRICE1,LPrice);
    }
  }

      ObjectSet(objname, OBJPROP_FONTSIZE, FSize);
  ObjectSetText(objname, s, FSize, Font, c);
} 

//--------------------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------------------
//
//

void RemoveObjects(string Pref)
{   
    int i;
    string objname = "";
    
    //
    //
    //
    //
    //
    
    for (i = ObjectsTotal(); i >= 0; i--) 
    {
       objname = ObjectName(i);
       if (StringFind(objname, Pref, 0) == 0) ObjectDelete(objname);
    }
}